<?xml version="1.0"?>

<!--
========================================================================
XSLT Stylesheet to convert DocBook XML into HTML
Copyright 2000 O'Reilly and Associates

Function: Converts DocBook books into formatted HTML files for easy
          viewing.
Input: DocBook Lite XML
Output: HTML files

Style: Each chapter is a single page
========================================================================
-->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xt="http://www.jclark.com/xt"
                extension-element-prefixes="xt"
		version="1.0">

<!--
========================================================================
                          HIERARCHICAL ELEMENTS
========================================================================
-->


<!--
match: book
===========
This template is the outermost container for the document.
It sets up a book file (index.html) and front matter files
(copyrght.htm, colophon.htm).

-->
<xsl:template match="book">
  <xt:document href="html/index.html" method="html">
    <xsl:fallback>Error while creating file.</xsl:fallback>
    <html>
      <head>
        <title>
          <xsl:value-of select="title"/>
        </title>
        <link rel="stylesheet" type="text/css" href="../style/style1.css"/>
      </head>
      <body>

        <h1 class="book"><xsl:value-of select="title"/></h1>

        <!-- Book info -->
        <p class="bookinfo">
          <xsl:text>by </xsl:text>
          <xsl:apply-templates select="author"/>
        </p>

        <!-- TOC -->
        <h3 class="tochead">Table of Contents</h3>
	<xsl:call-template name="toc"/>

      </body>
    </html>

    <xsl:apply-templates select="preface"/>
    <xsl:apply-templates select="chapter"/>
    <xsl:apply-templates select="appendix"/>

  </xt:document>
</xsl:template>


<!--
toc
===
Build a list of high-level elements

-->
<xsl:template name="toc">

  <xsl:if test="preface">
    <p class="toc">
      <a href="pref.html">Preface</a>
    </p>
  </xsl:if>

  <p class="toc">
    <xsl:for-each select="chapter"/>
      <a>
        <xsl:attribute name="href">
	  <xsl:text>ch</xsl:text>
          <xsl:number value="position()" format="01"/>
	  <xsl:text>.html</xsl:text>
        </xsl:attribute>
        <xsl:text>Chapter </xsl:text>
        <xsl:value-of select="position()"/>
        <xsl:text>: </xsl:text>
        <i><xsl:value-of select="title"/></i>
      </a>
      <br/>
    </xsl:for-each>
  </p>

  <xsl:if test="count(appendix)>0">
    <p class="toc">
      <xsl:for-each select="appendix"/>
        <a>
          <xsl:attribute name="href">
            <xsl:text>app</xsl:text>
            <xsl:number value="position()" format="a"/>
	    <xsl:text>.html</xsl:text>
          </xsl:attribute>
          <xsl:text>Appendix </xsl:text>
          <xsl:number value="position()" format="A"/>
          <xsl:text>: </xsl:text>
          <i><xsl:value-of select="title"/></i>
        </a>
        <br/>
      </xsl:for-each>
    </p>
  </xsl:if>
</xsl:template>


<!--
match: appendix
===============
Creates a file for an appendix of the form "appx.html"
where x is the appendix letter.

-->
<xsl:template match="appendix">
  <xsl:variable name="app">
    <xsl:number value="count(preceding::appendix)+1" format="a"/>
  </xsl:variable>
  <xt:document href="app{$app}.html" method="html">
    <xsl:fallback>Error while creating file.</xsl:fallback>
    <xsl:call-template name="file"/>
  </xt:document>
</xsl:template>


<!--
match: chapter
==============
Creates a file for a chapter of the form "chXX.htm"
where XX is the number of the chapter (01, 02, etc.).

-->
<xsl:template match="chapter">
  <xsl:variable name="chap">
    <xsl:number value="count(preceding::chapter)+1" format="01"/>
  </xsl:variable>
  <xt:document href="ch{$chap}.html" method="html">
    <xsl:fallback>Error while creating file.</xsl:fallback>
    <xsl:call-template name="file"/>
  </xt:document>
</xsl:template>


<!--
match: preface
==============
Creates a file for a preface.

-->
<xsl:template match="preface">
  <xt:document href="pref.html" method="html">
    <xsl:fallback>Error while creating file.</xsl:fallback>
    <xsl:call-template name="file"/>
  </xt:document>
</xsl:template>


<!--
match: sect1, sect2, sect3
==========================
Create an anchor for linking, then process the contents.

-->
<xsl:template match="sect1|sect2|sect3">
  <xsl:call-template name="drop-anchor"/>
  <xsl:apply-templates/>
</xsl:template>


<!--
drop-anchor
===========
Place an anchor just before the object if it has an ID attribute.

-->
<xsl:template name="drop-anchor">
  <xsl:if test="@id">
    <a>
      <xsl:attribute name="name">
        <xsl:value-of select="@id"/>
      </xsl:attribute>
    </a>
  </xsl:if>
</xsl:template>


<!--
========================================================================
                              HEADS
========================================================================
-->


<xsl:template match="book/title">
  <h1 class="book"><xsl:apply-templates/></h1>
</xsl:template>


<xsl:template match="appendix/title">
  <h1 class="chapter">
    <xsl:text>Appendix </xsl:text>
    <xsl:number value="count(preceding::appendix)+1" format="A"/>
    <xsl:text>.  </xsl:text>
    <xsl:apply-templates/>
  </h1>
</xsl:template>


<xsl:template match="chapter/title">
  <h1 class="chapter">
    <xsl:text>Chapter </xsl:text>
    <xsl:value-of select="count(preceding::chapter)+1"/>
    <xsl:text>.  </xsl:text>
    <xsl:apply-templates/>
  </h1>
</xsl:template>


<xsl:template match="part/title">
  <h1 class="chapter">
    <xsl:text>Part </xsl:text>
    <xsl:value-of select="count(preceding::part)+1"/>
    <xsl:text>.  </xsl:text>
    <xsl:apply-templates/>
  </h1>
</xsl:template>


<xsl:template match="preface/title">
  <h1 class="chapter">Preface</h1>
</xsl:template>


<xsl:template match="sect1/title">
  <h2 class="sect1">
    <xsl:choose>
      <xsl:when test="ancestor::appendix">
        <xsl:number value="count(preceding::appendix)+1" format="A"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="count(preceding::chapter)+1"/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>.</xsl:text>
    <xsl:number value="count(../preceding-sibling::sect1)+1"/>
    <xsl:text>. </xsl:text>
    <xsl:apply-templates/>
  </h2>
</xsl:template>


<xsl:template match="sect2/title">
  <h3 class="sect2">
    <xsl:choose>
      <xsl:when test="ancestor::appendix">
        <xsl:number value="count(preceding::appendix)+1" format="A"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="count(preceding::chapter)+1"/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>.</xsl:text>
    <xsl:number value="count(../../preceding-sibling::sect1)+1"/>
    <xsl:text>.</xsl:text>
    <xsl:number value="count(../preceding-sibling::sect2)+1"/>
    <xsl:text>. </xsl:text>
    <xsl:apply-templates/>
  </h3>
</xsl:template>


<xsl:template match="sect3/title">
  <h3 class="sect3">
    <xsl:choose>
      <xsl:when test="ancestor::appendix">
        <xsl:number value="count(preceding::appendix)+1" format="A"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="count(preceding::chapter)+1"/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>.</xsl:text>
    <xsl:number value="count(../../../preceding-sibling::sect1)+1"/>
    <xsl:text>.</xsl:text>
    <xsl:number value="count(../../preceding-sibling::sect2)+1"/>
    <xsl:text>.</xsl:text>
    <xsl:number value="count(../preceding-sibling::sect3)+1"/>
    <xsl:text>. </xsl:text>
    <xsl:apply-templates/>
  </h3>
</xsl:template>


<xsl:template match="appendix//figure/title">
  <h4 class="objtitle">
    <xsl:text>Figure </xsl:text>
    <xsl:number value-of="count(preceding::appendix)+1" format="A"/>
    <xsl:text>-</xsl:text>
    <xsl:number
      value-of="count(preceding::figure) -
                count(ancestor::appendix/preceding::figure) + 1"/>
    <xsl:text>. </xsl:text>
    <xsl:apply-templates/>
  </h4>
</xsl:template>


<xsl:template match="chapter//figure/title">
  <h4 class="objtitle">
    <xsl:text>Figure </xsl:text>
    <xsl:number value-of="count(preceding::chapter)+1"/>
    <xsl:text>-</xsl:text>
    <xsl:number
      value-of="count(preceding::figure) -
                count(ancestor::chapter/preceding::figure) + 1"/>
    <xsl:text>. </xsl:text>
    <xsl:apply-templates/>
  </h4>
</xsl:template>


<xsl:template match="appendix//table/title">
  <h4 class="objtitle">
    <xsl:text>Table </xsl:text>
    <xsl:number value-of="count(preceding::appendix)+1" format="A"/>
    <xsl:text>-</xsl:text>
    <xsl:number
      value-of="count(preceding::table) -
                count(ancestor::appendix/preceding::table) + 1"/>
    <xsl:text>. </xsl:text>
    <xsl:apply-templates/>
  </h4>
</xsl:template>


<xsl:template match="chapter//table/title">
  <h4 class="objtitle">
    <xsl:text>Table </xsl:text>
    <xsl:number value-of="count(preceding::chapter)+1"/>
    <xsl:text>-</xsl:text>
    <xsl:number
      value-of="count(preceding::table) -
                count(ancestor::chapter/preceding::table) + 1"/>
    <xsl:text>. </xsl:text>
    <xsl:apply-templates/>
  </h4>
</xsl:template>


<xsl:template match="title">
  <h4 class="objtitle">
    <xsl:apply-templates/>
  </h4>
</xsl:template>


<!--
========================================================================
                          BLOCK ELEMENTS
========================================================================
-->


<xsl:template match="author">
  <xsl:value-of select="."/>
</xsl:template>


<xsl:template match="blockquote">
  <blockquote class="node">
    <xsl:apply-templates/>
  </blockquote>
</xsl:template>


<xsl:template match="comment">
  <em class="comment"><xsl:apply-templates/></em>
</xsl:template>


<xsl:template match="para">
  <p>
    <xsl:apply-templates/>
  </p>
  <xsl:if test="not(ancestor::table)">
    <xsl:for-each select=".//footnote">
      <blockquote class="footnote">
        <xsl:apply-templates/>
      </blockquote>
    </xsl:for-each>
  </xsl:if>
</xsl:template>


<xsl:template match="programlisting">
  <blockquote>
    <pre class="programlisting">
      <xsl:apply-templates/>
    </pre>
  </blockquote>
</xsl:template>


<xsl:template match="figure">
  <xsl:call-template name="drop-anchor"/>
  <div class="figure">
    <xsl:apply-templates select="graphic"/>
  </div>
  <xsl:apply-templates select="title"/>
</xsl:template>


<xsl:template match="graphic">
  <img alt="figure">
    <xsl:attribute name="src">
      <xsl:value-of select="@fileref"/>
    </xsl:attribute>
  </img>
</xsl:template>


<xsl:template match="note">
  <xsl:call-template name="drop-anchor"/>
  <blockquote class="note">
    <xsl:if test="not( title )">
      <h4 class="objtitle">
        <xsl:if test="self::note">
          <xsl:text>NOTE</xsl:text>
        </xsl:if>
      </h4>
    </xsl:if>
    <xsl:apply-templates/>
  </blockquote>
</xsl:template>


<!--
========================================================================
                            FOOTNOTES
========================================================================
-->


<xsl:template match="footnote">
  <a>
    <xsl:attribute name="href">
      <xsl:text>#FOOTNOTE-</xsl:text>
      <xsl:number value="count( preceding::footnote ) +1"/>
    </xsl:attribute>
    <xsl:text>[note]</xsl:text>
  </a>
</xsl:template>


<xsl:template match="footnote/para">
  <p>
  <a>
    <xsl:attribute name="name">
      <xsl:text>FOOTNOTE-</xsl:text>
      <xsl:number value="count( preceding::footnote ) +1"/>
    </xsl:attribute>
  </a>
    <xsl:if test="count( preceding-sibling::para ) =0">
      <xsl:text>[</xsl:text>
      <xsl:number value="count( preceding::footnote ) +1"/>
      <xsl:text>]</xsl:text>
    </xsl:if>
    <xsl:apply-templates/>
  </p>
</xsl:template>


<!--
========================================================================
                             XREFs
========================================================================
-->


<xsl:template match="xref">
  <xsl:variable name="ident">
    <xsl:value-of select="@linkend"/>
  </xsl:variable>

  <xsl:for-each select="//*[@id=$ident]">
    <a>
      <xsl:attribute name="href">
	<xsl:choose>
          <xsl:when test="ancestor::appendix">
            <xsl:text>app</xsl:text>
            <xsl:number value="count( preceding::appendix )+1"
                        format="a"/>
            <xsl:text>.html</xsl:text>
          </xsl:when>
          <xsl:when test="ancestor::chapter">
            <xsl:text>ch</xsl:text>
            <xsl:number value="count( preceding::chapter )+1"
                        format="01"/>
            <xsl:text>.html</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <xsl:text>pref.html</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
        <xsl:text>#</xsl:text>
        <xsl:value-of select="$ident"/>
      </xsl:attribute>

      <xsl:choose>
        <xsl:when test="self::appendix">
          <xsl:text>Appendix </xsl:text>
          <xsl:number value="count( preceding::appendix )+1" format="A"/>
          <xsl:text>, "</xsl:text>
          <xsl:value-of select="title"/>
          <xsl:text>"</xsl:text>
        </xsl:when>

        <xsl:when test="self::chapter">
          <xsl:text>Chapter </xsl:text>
          <xsl:value-of select="count(preceding::chapter)+1"/>
          <xsl:text>, "</xsl:text>
          <xsl:value-of select="title"/>
          <xsl:text>"</xsl:text>
        </xsl:when>

        <xsl:when test="self::preface">
          <xsl:text>the Preface</xsl:text>
        </xsl:when>

        <xsl:when test="self::figure">
          <xsl:text>Figure </xsl:text>
          <xsl:choose>
            <xsl:when test="ancestor::appendix">
              <xsl:number value="count( preceding::appendix ) +1"
                                 format="A"/>
              <xsl:text>-</xsl:text>
              <xsl:number
                value="count( preceding::figure ) -
                       count( ancestor::appendix/preceding::figure ) +1"/>
            </xsl:when>
            <xsl:when test="ancestor::chapter">
              <xsl:number value="count( preceding::chapter ) +1"/>
              <xsl:text>-</xsl:text>
              <xsl:number
                value="count( preceding::figure ) -
                       count( ancestor::chapter/preceding::figure ) +1"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>0</xsl:text>
              <xsl:text>-</xsl:text>
              <xsl:number value="count( preceding::figure ) +1"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>

        <xsl:when test="self::table">
          <xsl:text>Table </xsl:text>
          <xsl:choose>
            <xsl:when test="ancestor::appendix">
              <xsl:number value="count( preceding::appendix ) +1"
                                 format="A"/>
              <xsl:text>-</xsl:text>
              <xsl:number
                value="count( preceding::table ) -
                       count( ancestor::appendix/preceding::table ) +1"/>
            </xsl:when>
            <xsl:when test="ancestor::chapter">
              <xsl:number value="count( preceding::chapter ) +1"/>
              <xsl:text>-</xsl:text>
              <xsl:number
                value="count( preceding::table ) -
                       count( ancestor::chapter/preceding::table ) +1"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:text>0</xsl:text>
              <xsl:text>-</xsl:text>
              <xsl:number value="count( preceding::table ) +1"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:when>

        <xsl:otherwise>
          <xsl:text>{XREF}</xsl:text>
        </xsl:otherwise>

      </xsl:choose>
    </a>
  </xsl:for-each>
</xsl:template>


<!--
filename
========
Return the filename that contains the current node.

-->
<xsl:template name="filename">
  <xsl:choose>
    <xsl:when test="ancestor::appendix">
      <xsl:text>app</xsl:text>
      <xsl:number value="count(preceding::appendix)+1" format="a"/>
      <xsl:text>.htm</xsl:text>
    </xsl:when>
    <xsl:when test="ancestor::chapter">
      <xsl:text>ch</xsl:text>
      <xsl:number value="count(preceding::chapter)+1" format="01"/>
      <xsl:text>.htm</xsl:text>
    </xsl:when>
    <xsl:when test="ancestor::preface">
      <xsl:text>pref.htm</xsl:text>
    </xsl:when>
  </xsl:choose>
</xsl:template>


<!--
========================================================================
                             LISTS
========================================================================
-->


<xsl:template match="orderedlist">
  <ol><xsl:apply-templates/></ol>
</xsl:template>


<xsl:template match="orderedlist/listitem">
  <li><xsl:apply-templates/></li>
</xsl:template>


<xsl:template match="itemizedlist">
  <ul><xsl:apply-templates/></ul>
</xsl:template>


<xsl:template match="itemizedlist/listitem">
  <li><xsl:apply-templates/></li>
</xsl:template>


<xsl:template match="variablelist">
  <dl><xsl:apply-templates/></dl>
</xsl:template>


<xsl:template match="varlistentry">
  <xsl:apply-templates/>
</xsl:template>


<xsl:template match="varlistentry/term">
  <dt><b><xsl:apply-templates/></b></dt>
</xsl:template>


<xsl:template match="varlistentry/listitem">
  <dd><xsl:apply-templates/></dd>
</xsl:template>

<!--
========================================================================
                            TABLES
========================================================================
-->


<!--
match: table, informaltable
===========================
Convert an XML table into an HTML table.

-->
<xsl:template match="table|informaltable">
  <xsl:call-template name="drop-anchor"/>
  <xsl:if test="self::table">
    <xsl:apply-templates select="title"/>
  </xsl:if>
  <table border="1">
    <xsl:apply-templates select="tgroup"/>
  </table>
  <xsl:for-each select=".//footnote">
    <blockquote class="footnote">
      <xsl:apply-templates/>
    </blockquote>
  </xsl:for-each>
</xsl:template>


<xsl:template match="tgroup|thead|tbody">
  <xsl:apply-templates/>
</xsl:template>


<xsl:template match="row">
  <tr><xsl:apply-templates/></tr>
</xsl:template>


<xsl:template match="thead//entry">
  <th><xsl:apply-templates/></th>
</xsl:template>


<xsl:template match="entry">
  <td>
    <xsl:if test="@spanname">
      <xsl:attribute name="colspan">
        <xsl:call-template name="getspan">
          <xsl:with-param name="spanname">
            <xsl:value-of select="@spanname"/>
          </xsl:with-param>
        </xsl:call-template>
      </xsl:attribute>
    </xsl:if>
    <xsl:if test="@morerows">
      <xsl:attribute name="rowspan">
        <xsl:value-of select="@morerows"/>
      </xsl:attribute>
    </xsl:if>
    <xsl:apply-templates/>
  </td>
</xsl:template>


<xsl:template name="getspan">
  <xsl:param name="spanname" value="span"/>
  <xsl:variable name="colstart">
    <xsl:for-each
      select="ancestor::tgroup/spanspec[@spanname='$spanname']">
      <xsl:value-of select="@namest"/>
    </xsl:for-each>
  </xsl:variable>
  <xsl:variable name="colend">
    <xsl:for-each
      select="ancestor::tgroup/spanspec[@spanname='$spanname']">
      <xsl:value-of select="@nameend"/>
    </xsl:for-each>
  </xsl:variable>
  <xsl:variable name="colstartnum">
    <xsl:for-each select="ancestor::tgroup/colspec[@colname='$colstart']">
      <xsl:value-of select="@colnum"/>
    </xsl:for-each>
  </xsl:variable>
  <xsl:variable name="colendnum">
    <xsl:for-each select="ancestor::tgroup/colspec[@colname='$colend']">
      <xsl:value-of select="@colnum"/>
    </xsl:for-each>
  </xsl:variable>
  <xsl:value-of select="$colendnum - $colstartnum"/>
</xsl:template>


<!--
========================================================================
                          INLINE ELEMENTS
========================================================================
-->


<xsl:template match="acronym">
  <span class="acronym"><xsl:apply-templates/></span>
</xsl:template>


<xsl:template match="application">
  <em class="application"><xsl:apply-templates/></em>
</xsl:template>


<xsl:template match="citetitle">
  <em class="citetitle"><xsl:apply-templates/></em>
</xsl:template>


<xsl:template match="command">
  <tt class="command"><xsl:apply-templates/></tt>
</xsl:template>


<xsl:template match="emphasis">
  <em class="emphasis"><xsl:apply-templates/></em>
</xsl:template>

<xsl:template match="emphasis[@role='bold']">
  <b class="emphasis-bold"><xsl:apply-templates/></b>
</xsl:template>


<xsl:template match="filename">
  <em class="filename"><xsl:apply-templates/></em>
</xsl:template>


<xsl:template match="firstterm">
  <em class="firstterm"><xsl:apply-templates/></em>
</xsl:template>


<xsl:template match="function">
  <tt class="function"><xsl:apply-templates/></tt>
</xsl:template>


<xsl:template match="literal">
  <tt class="literal"><xsl:apply-templates/></tt>
</xsl:template>


<xsl:template match="quote">
  <xsl:text>"</xsl:text>
  <xsl:apply-templates/>
  <xsl:text>"</xsl:text>
</xsl:template>


<xsl:template match="sgmltag">
  <tt class="sgmltag-element">
  <xsl:text><</xsl:text>
  <xsl:apply-templates/>
  <xsl:text>></xsl:text>
  </tt>
</xsl:template>


<xsl:template match="symbol">
  <span class="symbol"><xsl:apply-templates/></span>
</xsl:template>


<xsl:template match="systemitem[@class='url']">
  <a>
    <xsl:attribute name="href">
      <xsl:value-of select="."/>
    </xsl:attribute>
    <xsl:value-of select="."/>
  </a>
</xsl:template>


<!--
========================================================================
                          INDEXTERMS
========================================================================
-->


<xsl:template match="indexterm">
  <a>
    <xsl:attribute name="name">
      <xsl:text>INDEX-</xsl:text>
      <xsl:value-of select="@number"/>
    </xsl:attribute>
  </a>
</xsl:template>


<!--
========================================================================
                          FILE TEMPLATES
========================================================================
-->


<!--
file
====
This template generates a new HTML file to contain a chapter, part,
preface, or appendix.

-->
<xsl:template name="file">
  <html>
    <head>
      <title><xsl:value-of select="title"/></title>
      <link rel="stylesheet" type="text/css" href="../style/style1.css"/>
    </head>
    <body>
      <!-- Top Nav Bar -->

      <xsl:call-template name="nav-bar"/>
      <hr width="515" align="left"/>

      <!-- Body content -->

      <xsl:apply-templates/>

      <!-- Bottom Nav Bar -->

      <hr width="515" align="left"/>
      <xsl:call-template name="nav-bar"/>
    </body>
  </html>
</xsl:template>


<!--
nav-bar
=======
Create a set of three navigation buttons: previous, up, and next.

-->
<xsl:template name="nav-bar">
  <div class="navbar">
    <table width="515" border="0">
      <tr>
        <td align="left" valign="top" width="172">
          <a>
            <xsl:attribute name="href">
              <xsl:call-template name="prev-link"/>
            </xsl:attribute>
            <xsl:text><-- Previous</xsl:text>
          </a>
        </td>
        <td align="center" valign="top" width="171">
          <a href="index.html">
            <xsl:text>TOC</xsl:text>
          </a>
        </td>
        <td align="right" valign="top" width="172">
          <a>
            <xsl:attribute name="href">
              <xsl:call-template name="next-link"/>
            </xsl:attribute>
            <xsl:text>Next --></xsl:text>
          </a>
        </td>
      </tr>
    </table>
  </div>
</xsl:template>


<xsl:template name="prev-link">
  <xsl:choose>
    <xsl:when test="preceding-sibling::appendix">
      <xsl:text>app</xsl:text>
      <xsl:number value="count(preceding-sibling::appendix)" format="a"/>
      <xsl:text>.html</xsl:text>
    </xsl:when>
    <xsl:when test="preceding-sibling::chapter">
      <xsl:text>ch</xsl:text>
      <xsl:number value="count(preceding-sibling::chapter)" format="01"/>
      <xsl:text>.html</xsl:text>
    </xsl:when>
    <xsl:when test="preceding-sibling::preface">
      <xsl:text>pref.html</xsl:text>
    </xsl:when>
    <xsl:otherwise>
      <xsl:text>index.html</xsl:text>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<xsl:template name="next-link">
  <xsl:choose>
    <xsl:when test="self::preface">
      <xsl:text>ch01.html</xsl:text>
    </xsl:when>
    <xsl:when test="self::chapter and following-sibling::chapter">
      <xsl:text>ch</xsl:text>
      <xsl:number value="count(preceding-sibling::chapter)+2" format="01"/>
      <xsl:text>.html</xsl:text>
    </xsl:when>
    <xsl:when test="self::preface and following-sibling::chapter">
      <xsl:text>ch01.html</xsl:text>
    </xsl:when>
    <xsl:when test="self::chapter and following-sibling::appendix">
      <xsl:text>appa.html</xsl:text>
    </xsl:when>
    <xsl:when test="self::appendix and following-sibling::appendix">
      <xsl:text>app</xsl:text>
      <xsl:number value="count(preceding-sibling::appendix)+2"
                         format="a"/>
      <xsl:text>.html</xsl:text>
    </xsl:when>
    <xsl:otherwise>
      <xsl:text>index.html</xsl:text>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>
</xsl:stylesheet>
